import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { useState } from 'react';
import AsyncSelect from 'react-select/async';
import Select from 'react-select';
import { useClassName } from '../hooks/className';
import { debounce, useRequest } from '../tools';
export const FormFieldInputComp = (props) => {
    const { name, placeholder, required, htmlType, locked, autocomplete } = props;
    return (_jsxs("div", { className: useClassName(props), children: [_jsx(Label, { ...props }), _jsx("input", { type: htmlType, className: useClassName(props, { el: 'input' }), defaultValue: props.initial, id: inputId(props), name: name, required: required, disabled: locked, placeholder: placeholder, autoComplete: autocomplete, "aria-describedby": descId(props) }), _jsx(ErrorDescription, { ...props })] }));
};
export const FormFieldTextareaComp = (props) => {
    const { name, placeholder, required, locked, rows, cols } = props;
    return (_jsxs("div", { className: useClassName(props), children: [_jsx(Label, { ...props }), _jsx("textarea", { className: useClassName(props, { el: 'textarea' }), defaultValue: props.initial, id: inputId(props), rows: rows, cols: cols, name: name, required: required, disabled: locked, placeholder: placeholder, "aria-describedby": descId(props) }), _jsx(ErrorDescription, { ...props })] }));
};
export const FormFieldBooleanComp = (props) => {
    const { name, required, locked } = props;
    return (_jsxs("div", { className: useClassName(props), children: [_jsx(Label, { ...props }), _jsx("input", { type: "checkbox", className: useClassName(props, { el: 'input' }), defaultChecked: !!props.initial, id: inputId(props), name: name, required: required, disabled: locked, "aria-describedby": descId(props) }), _jsx(ErrorDescription, { ...props })] }));
};
export const FormFieldFileComp = (props) => {
    const { name, required, locked, multiple, accept } = props;
    return (_jsxs("div", { className: useClassName(props), children: [_jsx(Label, { ...props }), _jsx("input", { type: "file", className: useClassName(props, { el: 'input' }), id: inputId(props), name: name, required: required, disabled: locked, multiple: multiple ?? false, accept: accept }), _jsx(ErrorDescription, { ...props })] }));
};
// cheat slightly and match bootstrap 😱
// TODO make this configurable as an argument to `FastUI`
const styles = {
    control: (base) => ({ ...base, borderRadius: '0.375rem', border: '1px solid #dee2e6' }),
};
export const FormFieldSelectComp = (props) => {
    if (props.vanilla) {
        return _jsx(FormFieldSelectVanillaComp, { ...props });
    }
    else {
        return _jsx(FormFieldSelectReactComp, { ...props });
    }
};
export const FormFieldSelectVanillaComp = (props) => {
    const { name, required, locked, options, multiple, initial, placeholder, onChange } = props;
    const className = useClassName(props);
    const classNameSelect = useClassName(props, { el: 'select' });
    return (_jsxs("div", { className: className, children: [_jsx(Label, { ...props }), _jsxs("select", { id: inputId(props), className: classNameSelect, defaultValue: initial, multiple: multiple, name: name, required: required, disabled: locked, "aria-describedby": descId(props), placeholder: placeholder, onChange: () => onChange && onChange(), children: [multiple ? null : _jsx("option", {}), options.map((option, i) => (_jsx(SelectOptionComp, { option: option }, i)))] }), _jsx(ErrorDescription, { ...props })] }));
};
export const FormFieldSelectReactComp = (props) => {
    const { name, required, locked, options, multiple, initial, placeholder, onChange } = props;
    const className = useClassName(props);
    const classNameSelectReact = useClassName(props, { el: 'select-react' });
    let value;
    if (Array.isArray(initial)) {
        value = findDefaultArray(options, initial);
    }
    else {
        value = findDefault(options, initial);
    }
    const reactSelectOnChanged = () => {
        // TODO this is a hack to wait for the input to be updated, can we do better?
        setTimeout(() => {
            onChange && onChange();
        }, 50);
    };
    return (_jsxs("div", { className: className, children: [_jsx(Label, { ...props }), _jsx(Select, { id: inputId(props), classNamePrefix: "fastui-react-select", className: classNameSelectReact, isMulti: multiple ?? false, isClearable: true, defaultValue: value, name: name, required: required, isDisabled: locked, options: options, "aria-describedby": descId(props), styles: styles, placeholder: placeholder, onChange: reactSelectOnChanged }), _jsx(ErrorDescription, { ...props })] }));
};
const SelectOptionComp = ({ option }) => {
    if ('options' in option) {
        return (_jsx("optgroup", { label: option.label, children: option.options.map((o) => (_jsx(SelectOptionComp, { option: o }, o.value))) }));
    }
    else {
        return _jsx("option", { value: option.value, children: option.label });
    }
};
function findDefaultArray(options, value) {
    const foundValues = value.map((v) => findDefault(options, v));
    return foundValues.filter((v) => v);
}
function findDefault(options, value) {
    for (const option of options) {
        if ('options' in option) {
            const found = findDefault(option.options, value);
            if (found) {
                return found;
            }
        }
        else if (option.value === value) {
            return option;
        }
    }
}
export const FormFieldSelectSearchComp = (props) => {
    const { name, required, locked, searchUrl, initial, multiple, placeholder, onChange } = props;
    const [isLoading, setIsLoading] = useState(false);
    const request = useRequest();
    const loadOptions = debounce((inputValue, callback) => {
        setIsLoading(true);
        request({
            url: searchUrl,
            query: { q: inputValue },
        })
            .then(([, response]) => {
            const { options } = response;
            callback(options);
            setIsLoading(false);
        })
            .catch(() => {
            setIsLoading(false);
        });
    }, props.debounce ?? 300);
    const reactSelectOnChanged = () => {
        // TODO this is a hack to wait for the input to be updated, can we do better?
        setTimeout(() => {
            onChange && onChange();
        }, 50);
    };
    return (_jsxs("div", { className: useClassName(props), children: [_jsx(Label, { ...props }), _jsx(AsyncSelect, { classNamePrefix: "fastui-react-select", id: inputId(props), className: useClassName(props, { el: 'select-react' }), isMulti: multiple ?? false, cacheOptions: true, isClearable: true, defaultOptions: true, loadOptions: loadOptions, defaultValue: initial, noOptionsMessage: ({ inputValue }) => (inputValue ? 'No results' : 'Type to search'), name: name, required: required, isDisabled: locked, isLoading: isLoading, "aria-describedby": descId(props), styles: styles, placeholder: placeholder, onChange: reactSelectOnChanged }), _jsx(ErrorDescription, { ...props })] }));
};
const Label = (props) => {
    let { title } = props;
    if (!Array.isArray(title)) {
        title = [title];
    }
    return (_jsx("label", { htmlFor: inputId(props), className: useClassName(props, { el: 'label' }), children: title.map((t, i) => (_jsxs("span", { children: [i > 0 ? _jsx(_Fragment, { children: " \u203A" }) : null, " ", t] }, i))) }));
};
const inputId = (props) => `form-field-${props.name}`;
const descId = (props) => (props.description ? `${inputId(props)}-desc` : undefined);
const ErrorDescription = (props) => {
    const { description, error } = props;
    const descClassName = useClassName(props, { el: 'description' });
    const errorClassName = useClassName(props, { el: 'error' });
    return (_jsxs(_Fragment, { children: [description ? (_jsx("div", { id: descId(props), className: descClassName, children: description })) : null, error ? _jsx("div", { className: errorClassName, children: error }) : null] }));
};
